#include "stdafx.h"




namespace bbs{

bool extractURL(const std::wstring& text, std::vector<std::wstring>& url)
{
	using namespace boost::xpressive;

	try {
		wsregex http = wsregex::compile(L"(?:http|ttp|tp)(s)?$");
		wsregex rx = wsregex::compile(L"([a-zA-Z0-9_]*)(://[a-zA-Z0-9_/:%#$&?()~.=+-]+)");
		for (wsregex_iterator it(text.begin(), text.end(), rx), end; it != end; ++it)
		{
			std::wstring scheme = it->str(1);
			wsmatch what;
			if (regex_search(scheme, what, http))
			{
				url.push_back(L"http" + what.str(1) + it->str(2));
			}
		}
		return !url.empty();
	} catch (...) {}
	return false;
}


BBSOperator::BBSOperator(utl::SyncLog& log)
	: Logger(log, utl::Log::LOGTYPE_BBS)
{
}

BBSOperator::~BBSOperator()
{
	thread_.init.interrupt();
	thread_.execute.interrupt();
	thread_.reload.interrupt();
	thread_.dat.interrupt();
	thread_.create.interrupt();
	thread_.subject.interrupt();
	thread_.change.interrupt();
	thread_.post.interrupt();

	thread_.init.join();
	thread_.execute.join();
	thread_.reload.join();
	thread_.dat.join();
	thread_.create.join();
	thread_.subject.join();
	thread_.change.join();
	thread_.post.join();
}

void BBSOperator::initialize()
{
	thread_.init.interrupt();
	thread_.execute.interrupt();
	thread_.reload.interrupt();
	thread_.dat.interrupt();
	thread_.create.interrupt();
	thread_.subject.interrupt();
	thread_.change.interrupt();
	thread_.post.interrupt();

	bbs_.uninit();

	dat_.clear();
	subject_.clear();
	title_.clear();
	status_.clear();
	post_.clear();
	counter_.clear();
	links_.clear();

	callback_.update.get()(UPDATE_ALL);
}


bool BBSOperator::init(const std::wstring path)
{
	if (path_.get() == path) { return false; }
	path_ = path;
	board_.apply([](std::wstring& board){ board.clear(); });

	initialize();

	create();
	return true;
}

// R^NgURLǂݍ܂ĂȂƂɐVKURLǂݍ
bool BBSOperator::once(const std::wstring& board, const BoardControl& cfg)
{
	if (board_.func([](const std::wstring& board){ return !board.empty(); })) { return false; }
	execute(board, cfg);
	return true;
}

// URLȂXꗗ擾
// URLύXŃXbh
void BBSOperator::execute(const std::wstring& board, const BoardControl& cfg)
{
	thread_.execute.timed_join(0);
	if (thread_.execute.joinable()) { return; }

	boost::thread thr(&BBSOperator::executeThread, this, board, boost::cref(cfg));
	thread_.execute.swap(thr);
}

void BBSOperator::reload()
{
	thread_.reload.timed_join(0);
	if (thread_.reload.joinable()) { return; }

	boost::thread thr(&BBSOperator::reloadThread, this, true, false, false);
	thread_.reload.swap(thr);
}

void BBSOperator::terminate()
{
	thread_.dat.interrupt();
}

bool BBSOperator::teminated()
{
	thread_.dat.timed_join(0);
	if (thread_.dat.joinable()) { return false; }
	return true;
}

void BBSOperator::create(int diff)
{
	thread_.create.timed_join(0);
	if (thread_.create.joinable()) { return; }

	boost::thread thr(&BBSOperator::createThread, this, diff);
	thread_.create.swap(thr);
}

void BBSOperator::subject()
{
	thread_.subject.timed_join(0);
	if (thread_.subject.joinable()) { return; }

	boost::thread thr(&BBSOperator::subjectThread, this);
	thread_.subject.swap(thr);
}

void BBSOperator::change(const std::wstring& key)
{
	thread_.change.timed_join(0);
	if (thread_.change.joinable()) { return; }

	boost::thread thr(&BBSOperator::changeThread, this, key, cfg_.func([](const BoardControl& cfg) { return cfg.reload; }), true, false);
	thread_.change.swap(thr);
}

void BBSOperator::post(const std::wstring& name, const std::wstring& mail, const std::wstring& message)
{
	thread_.post.timed_join(0);
	if (thread_.post.joinable())
	{
		callback_.post.get()(false);
		callback_.update.get()(UPDATE_POST);
		return;
	}

	boost::thread thr(&BBSOperator::postThread, this, name, mail, message);
	thread_.post.swap(thr);
}

//---------------------------------------------------------------------------------

bool BBSOperator::getFastest(ThreadInfo& info)
{
	time_t org = 0;
	std::wstring key;
	if (getKey(key))
	{
		org = wcstoul(key.c_str(), NULL, 10);
	}

	int stop = 0;
	if (bbs_.getStop(stop))
	{
		return subject_.func([&](const std::vector<ThreadInfo>& subject)->bool{
			ThreadInfo x;
			for (auto it = subject.begin(); it != subject.end(); ++it)
			{
				if (it->count >= stop) { continue; }

				time_t time = wcstoul(it->key.c_str(), NULL, 10);
				if (time > org && it->speed > x.speed) { x = *it; }
			}

			if (x.key.empty())
			{
				return false;
			}
			info = x;
			return true;
		});
	}
	return false;
}

std::wstring BBSOperator::getBoardTitle()
{
	std::wstring title;
	if (!bbs_.getBoardTitle(title)) { return L"URLݒ"; }

	if (title.empty())
	{
		auto subject = subject_.get();
		if (subject && !subject->empty())
		{
			title = L"f^Cg擾s";
		}
		else
		{
			title = L"ΉURL";
		}
	}
	else
	{
		title = L"f [ " + title + L" ]";
	}

	return title;
}

std::wstring BBSOperator::getThreadTitle()
{
	std::wstring title;
	if (!bbs_.getThreadTitle(title)) { return L"URLݒ"; }

	if (!title.empty())
	{
		title = L"[ " + title + L" ]";
	}
	else
	{
		title = L"Xbh^Cg擾s";
	}

	int count = 0;
	if (bbs_.getCount(count))
	{
		std::wstring str;
		int pos = 0;
		if (bbs_.getPos(pos) && pos != 0 && pos != count)
		{
			str = (boost::wformat(L"%d/") % pos).str();
		}

		title += (boost::wformat(L" (%s%d)") % str.c_str() % count).str();
	}

	return title;
}

//---------------------------------------------------------------------------------

void BBSOperator::executeThread(const std::wstring& board, const BoardControl& cfg)
{
	thread_.init.interrupt();
	thread_.init.join();

	cfg_ = cfg;

	// URLu
	std::wstring url = board;
	cfg_.apply([&](const BoardControl& cfg) {
		for (auto it = cfg.url.begin(); it != cfg.url.end(); ++it)
		{
			(*it)(url);
		}
	});

	// URLȂXbhꗗXV
	if (!url.empty() && board_.func([&](const std::wstring& board) { return board == url; }))
	{
		subject();
		return;
	}

	board_ = url;

	initialize();

	boost::thread thr(&BBSOperator::initThread, this, url);
	thread_.init.swap(thr);
}

void BBSOperator::initThread(const std::wstring& board)
{
	title_ = L"ǂݍݒ";
	callback_.update.get()(UPDATE_ALL);

	std::wstring shitaraba = DEFAULT_BBS_SHITARABA;
	std::wstring userAgent = DEFAULT_BBS_USERAGENT;
	int timeout = DEFAULT_BBS_TIMEOUT;
	cfg_.apply([&](const BoardControl& cfg){ shitaraba = cfg.shitaraba; userAgent = cfg.userAgent; timeout = cfg.timeout; });
	if (bbs_.init(board, shitaraba, userAgent, timeout))
	{// f`ƌfݒǂݍ
		if (subjectThread())
		{// Xꗗǂݍ
			if (bbs_.hasKey())
			{// XbhL[
				if (cfg_.func([](const BoardControl& cfg){ return cfg.thread; }))
				{// Xbhړ
					if (bbs_.isStop())
					{
						ThreadInfo info;
						if (getFastest(info))
						{
							changeThread(info.key, cfg_.func([](const BoardControl& cfg) { return cfg.reload; }), true, true);
							return;
						}
					}
				}
			
				reloadThread(cfg_.func([](const BoardControl& cfg) { return cfg.reload; }), true, true);
				callback_.subject.get()(false);
				return;
			}


			if (cfg_.func([](const BoardControl& cfg){ return cfg.board; }))
			{// Xbhړ
				ThreadInfo info;
				if (getFastest(info))
				{
					changeThread(info.key, cfg_.func([](const BoardControl& cfg) { return cfg.reload; }), true, true);
					return;
				}
			}
		}

		// f^Cg
		title_ = getBoardTitle();

		callback_.dat.get()();
		callback_.subject.get()(false);
		callback_.update.get()(UPDATE_ALL);
		return;
	}

	title_ = L"ΉURL";

	callback_.dat.get()();
	callback_.subject.get()(false);
	callback_.update.get()(UPDATE_ALL);
}

// reload // XV
// stop // XȂ~
// move // XbhړL
void BBSOperator::datThread(bool reload, bool stop, bool move)
{
	status_ = L"ǂݍݒ";
	title_ = getThreadTitle();
	callback_.update.get()(UPDATE_ALL);
	bool flag = false; // XbhȊO true
	while (true)
	{
		std::wstring userAgent = DEFAULT_BBS_USERAGENT;
		int timeout = DEFAULT_BBS_TIMEOUT;
		cfg_.apply([&](const BoardControl& cfg){ userAgent = cfg.userAgent; timeout = cfg.timeout; });
		switch (bbs_.loadDat(userAgent, timeout))
		{
		case STATE_OK:
			{
				create();
				title_ = getThreadTitle();
				status_.clear();
				break;
			}
		case STATE_NOT_MODIFIED:
			{
				status_.clear();
				break;
			}

		case STATE_ERROR_CONNECT:
			{
				status_ = L"ڑs";
				break;
			}

		case STATE_ERROR_RECIEVE:
			{
				status_ = L"ǂݍݎs";
				break;
			}
		case STATE_ERROR_INVALID:
		case STATE_ERROR_NOKEY:
			{
				status_ = L"XbhIĂ";
				goto exit_loop;
				break;
			}

		default:
			{
				status_.clear();
				break;
			}
		}

		callback_.update.get()(UPDATE_STATUS);

		//addLog(L"reload:%s stop:%s move:%s flag:%s",
		//	reload ? L"true" : L"false",
		//	stop ? L"true" : L"false",
		//	move ? L"true" : L"false",
		//	flag ? L"true" : L"false");

		if (!bbs_.isStop())
		{
			flag = true;
		}
		else
		{
			if ((move || flag) && cfg_.func([](const BoardControl& cfg) { return cfg.move; }))
			{// Xbhړ
				if (subjectThread())
				{
					ThreadInfo info;
					if (getFastest(info))
					{
						counter_.clear();
						callback_.update.get()(UPDATE_STATUS);

						change(info.key);
						return;
					}
				}
			}
			else if (stop)
			{
				status_ = L"XbhI";
				break;
			}
		}

		// XV
		if (!reload) { break; }
		
		int interval = DEFAULT_BBS_INTERVAL;
		cfg_.apply([&](const BoardControl& cfg){ interval = cfg.interval; });
		for (int i = 0; i < interval; ++i)
		{
			counter_ = interval - i;
			callback_.update.get()(UPDATE_ALL);

			try { boost::this_thread::sleep(boost::posix_time::seconds(1)); }
			catch (...) { goto exit_loop; }
		}
	}

exit_loop:
	counter_.clear();
	callback_.update.get()(UPDATE_STATUS);
	return;
}

void BBSOperator::reloadThread(bool reload, bool stop, bool move)
{
	if (!bbs_.hasKey())
	{
		status_ = L"XbhIĂ";
		callback_.update.get()(UPDATE_STATUS);
		return;
	}

	thread_.dat.interrupt();
	thread_.dat.join();

	boost::thread thr(&BBSOperator::datThread, this, reload, stop, move);
	thread_.dat.swap(thr);
}

void BBSOperator::createThread(int diff)
{
	utl::HTMLEscape esc;
	utl::NameSplitter split;
	std::vector<ResInfo> dat;

	int res = DEFAULT_BBS_RES;
	cfg_.apply([&](const BoardControl& cfg) { res = cfg.res; });

	if (bbs_.getDat(dat, diff, res))
	{
		std::vector<ResInfo> tdat;
		std::vector<std::wstring> tlinks;
		for (auto it = dat.begin(); it != dat.end(); ++it)
		{
			ResInfo res;
			res.number = it->number;
			res.name = esc(it->name);
			split(it->name, res.split);
			res.mail = esc(it->mail);
			res.datetime = esc(it->datetime);

			// Xu
			std::wstring message = it->message;
			cfg_.apply([&](const BoardControl& cfg) {
				for (auto it = cfg.message.begin(); it != cfg.message.end(); ++it)
				{
					(*it)(message);
				}
			});
			res.message = esc(message);

			res.title = esc(it->title);
			res.id = esc(it->id);

			tdat.push_back(res);

			std::vector<std::wstring> links;
			if (extractURL(res.message, links))
			{
				tlinks.insert(tlinks.end(), links.begin(), links.end());
			}
		}
		dat_ = tdat;
		links_ = tlinks;
	}

	if (bbs_.hasKey())
	{
		title_ = getThreadTitle();
	}
	else
	{
		title_ = getBoardTitle();
	}
	callback_.update.get()(UPDATE_TITLE);
	callback_.dat.get()();
}

bool BBSOperator::subjectThread()
{
	callback_.subject.get()(true);

	std::wstring userAgent = DEFAULT_BBS_USERAGENT;
	int timeout = DEFAULT_BBS_TIMEOUT;
	cfg_.apply([&](const BoardControl& cfg){ userAgent = cfg.userAgent; timeout = cfg.timeout; });
	if (bbs_.loadSubject(userAgent, timeout) >= STATE_OK)
	{
		std::vector<ThreadInfo> subject;
		bbs_.getSubject(subject);
		subject_ = subject;
		callback_.subject.get()(false);
		return true;
	}
	callback_.subject.get()(false);
	return false;
}

void BBSOperator::changeThread(const std::wstring& key, bool reload, bool stop, bool move)
{
	if (!bbs_.change(key)) { return; }
	
	std::wstring url;
	if (bbs_.getURL(url))
	{
		board_ = url;
	}
	reloadThread(reload, stop, move);
	callback_.subject.get()(false);
}

void BBSOperator::postThread(const std::wstring& name, const std::wstring& mail, const std::wstring& message)
{
	//if (message.empty())
	//{
	//	post_ = L"{܂";
	//	callback_.post.get()(false);
	//	callback_.update.get()(UPDATE_POST);
	//	return;
	//}

	int code = 0;
	std::wstring error;

	post_ = L"ݒ";
	callback_.update.get()(UPDATE_ALL);

	std::wstring userAgent = DEFAULT_BBS_USERAGENT;
	int timeout = DEFAULT_BBS_TIMEOUT;
	cfg_.apply([&](const BoardControl& cfg){ userAgent = cfg.userAgent; timeout = cfg.timeout; });
	switch (bbs_.post(userAgent, timeout, name, mail, message, code, error))
	{
	case STATE_OK:
	case STATE_NOT_MODIFIED:
		{
			post_ = L"ݐ";
			callback_.post.get()(true);
			callback_.update.get()(UPDATE_POST);

			reloadThread(counter_.get() ? true : false, true, true);
			return;
		}
	case STATE_ERROR_CODE:
		{
			post_ = L"ݎs" + (boost::wformat(L" (%d)") % code).str();
			break;
		}
	case STATE_ERROR_CONNECT:
		{
			post_ = L"ڑs";
			break;
		}
	case STATE_ERROR_RECIEVE:
		{
			post_ = L"ǂݍݎs" + (boost::wformat(L" (%d)") % code).str();
			break;
		}
	case STATE_ERROR_POST:
		{
			post_ = L"ݎs" + (boost::wformat(L" (%d %s)") % code % error).str();
			break;
		}
	case STATE_ERROR_INVALID:
	case STATE_ERROR_NOKEY:
		{
			post_ = L"XbhIĂ";
			break;
		}
	default:
		{
			post_ = L"ݎs";
			break;
		}
	}

	callback_.post.get()(false);
	callback_.update.get()(UPDATE_POST);
}

}; // namespace bbs