#pragma once

#include "BBSManager.h"

#include <boost/thread.hpp>
#include <boost/thread/recursive_mutex.hpp>

#include <boost/functional.hpp>



#define DEFAULT_BBS_USERAGENT	L"Monazilla/1.00 (PCRPlayer/1.00)"
#define DEFAULT_BBS_TIMEOUT		10000
#define DEFAULT_BBS_SHITARABA	L"jbbs\\.shitaraba\\.net"
#define DEFAULT_BBS_INTERVAL	7
#define DEFAULT_BBS_RES			10

namespace bbs{

struct ReplaceElement {
	ReplaceElement()
		: enable(true)
	{}

	ReplaceElement(
		bool enable,
		const std::wstring& desc,
		const std::wstring& regex,
		const std::wstring& format)
		: enable(enable)
		, desc(desc)
		, regex(regex)
		, format(format)
	{}

	bool enable;
	std::wstring desc;
	std::wstring regex;
	std::wstring format;

	bool operator()(std::wstring& str) const
	{
		using namespace boost::xpressive;

		if (!enable) { return false; }

		if (str.empty() && regex.empty())
		{// ̐K\󕶎Ƀ}b`
			str = format;
			return true;
		}

		try {
			// regex_constants::single_line
			// $sɃ}b`ȂȂ
			str = boost::xpressive::regex_replace(str, wsregex::compile(regex, regex_constants::single_line), format);
			return true;
		} catch (...) {}
		return false;
	}
};

// fݒ
struct BoardControl {
	BoardControl()
		: userAgent(DEFAULT_BBS_USERAGENT)
		, timeout(DEFAULT_BBS_TIMEOUT)
		, shitaraba(DEFAULT_BBS_SHITARABA)
		, reload(true)
		, interval(DEFAULT_BBS_INTERVAL)
		, thread(true)
		, board(true)
		, move(true)
		, res(DEFAULT_BBS_RES)
	{
		typedef ReplaceElement RE;
		url.push_back(RE(true, L"΋AhXu", L"jbbs\\.livedoor\\.jp(/bbs/read\\.cgi)?", L"jbbs.shitaraba.net"));
		url.push_back(RE(false, L"URLu", L"^$", L"http://"));

		message.push_back(RE(true, L"󔒂ɒu", L"&#160;|&nbsp;", L" "));
		message.push_back(RE(true, L"̉s폜", L"^([ @]*<br>)+", L""));
		message.push_back(RE(true, L"̉s폜", L"(<br>[ @]*)+$", L""));
		message.push_back(RE(true, L"s̋󔒂폜", L"^[ @]|(<br>)[ @]", L"$1"));
		message.push_back(RE(true, L"sȏ̋sȗ", L"([ @]*<br>){2,}", L"<br><br>"));
		message.push_back(RE(false, L"s폜", L"([ @]*<br>)+", L"<br>"));
		message.push_back(RE(false, L"10s葽sȗ", L"((.*?<br>){10})(.*)", L"$1iȗ܂j"));
		message.push_back(RE(false, L"A邗ȗ", L"[wWv]{2,}(?![A-Za-z0-9_%&\\--/=])", L""));
	}

	std::wstring userAgent; // userAgent
	int timeout; // ^CAEg(ms)
	std::wstring shitaraba; // ΌfhC
	bool reload; // XbhXV
	int interval; // XbhXVԊu
	bool thread; // Xbhړ/XbhI
	bool board; // Xbhړ/f
	bool move; // Xbhړ/XbhI
	int res; // X\
	std::vector<ReplaceElement> url; // URLu
	std::vector<ReplaceElement> message; // X{u
};

// XVR[obNR}h
enum UPDATE {
	UPDATE_ALL,
	UPDATE_TITLE,
	UPDATE_STATUS,
	UPDATE_POST,
};

class BBSOperator : protected utl::Logger {
private:
	struct Callback {
		Callback()
			: update([](long){})
			, post([](bool){})
			, subject([](bool){})
			, dat([](){})
		{}

		utl::SyncValue<boost::function<void(long)> > update; // BBSo[XV(TITLE/STATUS/POST/ALL)
		utl::SyncValue<boost::function<void(bool)> > post; // BBSo[e(SUCCEEDED/FAILED)
		utl::SyncValue<boost::function<void(bool)> > subject; // XbhI_CAOXV(START/END)
		utl::SyncValue<boost::function<void()> > dat; // X|bvAbvXV
	};

	struct Thread {
		utl::SyncThread execute;
		utl::SyncThread reload;
		utl::SyncThread init;
		utl::SyncThread dat;
		utl::SyncThread create;
		utl::SyncThread subject;
		utl::SyncThread change;
		utl::SyncThread post;
	};

	utl::SyncValue<BoardControl> cfg_;

	utl::SyncValue<std::wstring> path_; // ĐĂ铮̃pX ύXꂽ烊Zbg
	utl::SyncValue<std::wstring> board_; // Xbh/fURL ̂ƂR^NgURLǂݍ

	Thread thread_;
	BBSManager bbs_;

	utl::SyncOptional<std::vector<ResInfo> > dat_;
	utl::SyncOptional<std::vector<ThreadInfo> > subject_;

	utl::SyncOptional<std::wstring> title_;
	utl::SyncOptional<std::wstring> status_;
	utl::SyncOptional<std::wstring> post_;
	utl::SyncOptional<int> counter_;
	utl::SyncOptional<std::vector<std::wstring> > links_;

	Callback callback_;

	void initialize();

public:
	BBSOperator(utl::SyncLog& log);
	virtual ~BBSOperator();

	operator bool() { return bbs_; }

public:
	void setConfig(const BoardControl& cfg) { cfg_ = cfg; create(); }

	void setUpdateCallback(const boost::function<void(int)>& callback) { callback_.update = callback; }
	void setPostCallback(const boost::function<void(bool)>& callback) { callback_.post = callback; }
	void setSubjectCallback(const boost::function<void(bool)>& callback) { callback_.subject = callback; }
	void setDatCallback(const boost::function<void()>& callback) { callback_.dat = callback; }

public:
	// t@Cςɏ
	bool init(const std::wstring path);
	// ݒǂݍ݂ƃXbhǂݍ
	bool once(const std::wstring& board, const BoardControl& cfg);
	void execute(const std::wstring& board, const BoardControl& cfg);
	void reload();
	void terminate();
	bool teminated();
	void create(int diff = 0);
	// ꗗXV
	void subject();
	// XbhύXƓǂݍ
	void change(const std::wstring& key);

	void post(const std::wstring& name, const std::wstring& mail, const std::wstring& message);

	

public:
	bool getDat(std::vector<ResInfo>& dat) { return dat_.get(dat); }
	bool getLinks(std::vector<std::wstring>& links) { return links_.get(links); }
	bool getSubject(std::vector<ThreadInfo>& subject) { return subject_.get(subject); }
	

public:
	bool getURL(std::wstring& url) { return bbs_.getURL(url); }
	bool getBoardURL(std::wstring& url) { return bbs_.getBoardURL(url); }
	bool getThreadURL(std::wstring& url) { return bbs_.getThreadURL(url); }
	bool getBoardTitle(std::wstring& title) { return bbs_.getBoardTitle(title); }
	bool getThreadTitle(std::wstring& title) { return bbs_.getThreadTitle(title); }
	bool getNoName(std::wstring& noname) { return bbs_.getNoName(noname); }
	bool getCount(int& count) { return bbs_.getCount(count); }
	bool getKey(std::wstring& key) { return bbs_.getKey(key); }
	bool hasKey() { return bbs_.hasKey(); }
	bool getStop(int& stop) { return bbs_.getStop(stop); }
	bool isStop() { return bbs_.isStop(); }

public:
	void setTitle(const std::wstring& title) { title_ = title; }
	bool getTitle(std::wstring& title) { return title_.get(title); }
	void setStatus(const std::wstring& status) { status_ = status; }
	bool getStatus(std::wstring& status) { return status_.get(status); }
	void setPost(const std::wstring& post) { post_ = post; }
	bool getPost(std::wstring& post) { return post_.get(post); }
	void clearPost() { post_.clear(); }
	bool getCounter(int& counter) { return counter_.get(counter); }
	bool hasCounter() { return static_cast<bool>(counter_.get()); }

private:
	bool getFastest(ThreadInfo& info);
	std::wstring getBoardTitle();
	std::wstring getThreadTitle();

private:
	void executeThread(const std::wstring& board, const BoardControl& cfg);
	void initThread(const std::wstring& board);
	void datThread(bool reload, bool stop, bool move);
	void reloadThread(bool reload, bool stop, bool move);
	void createThread(int diff);
	bool subjectThread();
	void changeThread(const std::wstring& key, bool reload, bool stop, bool move);
	void postThread(const std::wstring& name, const std::wstring& mail, const std::wstring& message);
};


}; // namespace bbs