#include "stdafx.h"

namespace dsp {

bool OnlineBuilder::render(HWND hwnd, UINT msg, const std::wstring& path)
{
	NetworkControl network;
	DecoderControl decoder;
	DecoderIDSet decoderSet;
	{
		boost::recursive_mutex::scoped_lock lock(mxcfg_);
		network = cfg_.network;
		decoder = cfg_.decoder;
		decoderSet = cfg_.decoderSet;
	}

	boost::recursive_mutex::scoped_lock lock(mutex_);

	text_ = L"\z";

	network::URLParser url(path);
	if (url)
	{
		SourcePtr source;
		VideoPtr video;
		AudioPtr audio;

		// FLV
		//DebugText(L"OnlineBuilder::URLBufferSource\n");
		source.reset(new URLBufferSource(network.reader.userAgent, network.reader.timeout));
		video.reset(new EVRCustom(decoder.dxva2));
		audio.reset(new DSR());
		if (graph_.renderEx(hwnd, path, source, video, audio, decoderSet, true))
		{
			text_.clear();
			return true;
		}

		// WMASFSourceɉȂURLnƒԒʐM҂ԂɂȂ邽ߔʂĂ
		//if (source->type() != SOURCE_TYPE_ASF)
		//{
		//	text_ = L"\zs";
		//	return false;
		//}

		if (source->isTimeout())
		{
			text_ = L"^CAEg";
			return false;
		}

		try { boost::this_thread::interruption_point(); }
		catch (...) { text_.clear(); return false; }

		// WMV
		//DebugText(L"OnlineBuilder::WMASFSource\n");
		source.reset(new WMASFSource());
		video.reset(new EVRCustom(decoder.dxva2));
		audio.reset(new DSR());
		if (graph_.render(hwnd, path, source, video, audio, decoderSet))
		{
			text_.clear();
			return true;
		}

		// ǂݍݎsɃ^CAEgߔp~
		//try { boost::this_thread::interruption_point(); }
		//catch(...) { text_.clear(); return false; }

		// UNKNOWN
		//DebugText(L"OnlineBuilder::URLFileSource\n");
		//source.reset(new URLFileSource());
		//video.reset(new EVRCustom());
		//audio.reset(new DSR());
		//if (graph_.renderEx(hwnd, path, source, video, audio))
		//{
		//	return true;
		//}
	}

	text_ = L"\zs";
	return false;
}


bool OnlineBuilder::onBufferEnter(HWND hwnd, UINT msg, const std::wstring& path)
{
	addLog(L"OnlineBuilder::onBufferEnter");

	rebuild_ = Rebuild();

	boost::recursive_mutex::scoped_lock lock(mutex_);
	buffer_.init = true;
	buffer_.enable = true;
	buffer_.count++;
	return true;
}

bool OnlineBuilder::onBufferExit(HWND hwnd, UINT msg, const std::wstring& path)
{
	addLog(L"OnlineBuilder::onBufferExit");

	rebuild_ = Rebuild();

	boost::recursive_mutex::scoped_lock lock(mutex_);
	buffer_.enable = false;
	return true;
}

bool OnlineBuilder::onEnterLoop(HWND hwnd, UINT msg, const std::wstring& path)
{
	rebuild_ = Rebuild();
	return true;
}

bool OnlineBuilder::onLoop(HWND hwnd, UINT msg, const std::wstring& path)
{
	bool enable = false;
	{
		boost::recursive_mutex::scoped_lock lock(mutex_);
		enable = buffer_.enable;
	}

	DWORD time = timeGetTime();

	//if (enable && getProgress(rebuild_.buffer.now))
	//{
	//	if (!rebuild_.buffer.init)
	//	{// JnԂL^ď
	//		rebuild_.buffer.old = rebuild_.buffer.now;
	//		rebuild_.buffer.time = time;
	//		rebuild_.buffer.init = true;
	//	}
	//	else
	//	{
	//		if (rebuild_.buffer.now != rebuild_.buffer.old || rebuild_.buffer.now >= 100)
	//		{// ił邩100%ȂXV
	//			rebuild_.buffer.old = rebuild_.buffer.now;
	//			rebuild_.buffer.time = time;
	//		}
	//		else if (time - rebuild_.buffer.time > 60000)
	//		{
	//			addLog(L"OnlineBuilder::onLoop rebuild (Buffer)");
	//			if (!onRebuild(hwnd, msg, path)) { return false; }
	//			return true;
	//		}
	//	}
	//}
	//else if (rebuild_.running.enable && isRunning())
	//{
	//	// ĐXe[^X݂ăOtRunĂ̂
	//	// _RunĂȂꍇɍĐڑ
	//	if (!rebuild_.running.init)
	//	{
	//		addLog(L"OnlineBuilder::onLoop run (Graph)");
	//		rebuild_.running.time = time;
	//		rebuild_.running.init = true;
	//	}
	//	else
	//	{
	//		if (isAudioRunning() && isVideoRunning())
	//		{
	//			addLog(L"OnlineBuilder::onLoop run (Audio/Video)");
	//			rebuild_.running.enable = false;
	//		}
	//		else if (!hasVideo() && isAudioRunning())
	//		{
	//			addLog(L"OnlineBuilder::onLoop run (Audio)");
	//			rebuild_.running.enable = false;
	//		}
	//		else if (!hasAudio() && isVideoRunning())
	//		{
	//			addLog(L"OnlineBuilder::onLoop run (Video)");
	//			rebuild_.running.enable = false;
	//		}
	//		else if (time - rebuild_.running.time > 60000)
	//		{
	//			addLog(L"OnlineBuilder::onLoop rebuild (Running)");
	//			if (!onRebuild(hwnd, msg, path)) { return false; }
	//			return true;
	//		}
	//	}
	//}
	//else if (getSampleTime(rebuild_.sample.now))
	//{
	//	if (rebuild_.sample.start == 0)
	//	{// JnԂL^ď
	//		rebuild_.sample.start = rebuild_.sample.old = rebuild_.sample.now;
	//		rebuild_.sample.time = time;
	//	}
	//	else
	//	{
	//		if (rebuild_.sample.now == rebuild_.sample.start)
	//		{// JnĂ܂iłȂ
	//			rebuild_.sample.time = time;
	//		}
	//		else
	//		{
	//			if (rebuild_.sample.now != rebuild_.sample.old)
	//			{// iłȂXV
	//				rebuild_.sample.old = rebuild_.sample.now;
	//				rebuild_.sample.time = time;
	//			}
	//			else if (time - rebuild_.sample.time > 60000)
	//			{
	//				addLog(L"OnlineBuilder::onLoop rebuild (Sample)");
	//				if (!onRebuild(hwnd, msg, path)) { return false; }
	//				return true;
	//			}
	//		}
	//	}
	//}

	int bitrate = 0;
	if (getCurrentBitrate(bitrate))
	{
		if (!rebuild_.bitrate.init)
		{
			rebuild_.bitrate.time = time;
			rebuild_.bitrate.init = true;
		}
		else
		{
			if (bitrate > 0)
			{
				rebuild_.bitrate.time = time;
			}
			else if (time - rebuild_.bitrate.time > 60000)
			{
				addLog(L"OnlineBuilder::onLoop rebuild (Bitrate)");
				if (!onRebuild(hwnd, msg, path)) { return false; }
				return true;
			}
		}
	}

	return true;
}

bool OnlineBuilder::onRebuild(HWND hwnd, UINT msg, const std::wstring& path)
{
	if (!build(hwnd, msg, path)) { text_.clear(); return false; }
	text_.clear();
	return true;
}

} // namespace dsp
