#pragma once

#include "PeerCast.h"

#include <boost/thread.hpp>
#include <boost/functional.hpp>

namespace network {


class PeerCastManager {
	struct Callback {
		Callback()
			: bbs([](){})
			, channel([](bool){})
		{}

		utl::SyncValue<boost::function<void()> > bbs;
		utl::SyncValue<boost::function<void(bool)> > channel;
	};

	PeerCast peca_;

	boost::thread_group group_;
	utl::SyncThread thread_;

	utl::SyncValue<ChannelInfo> info_;
	utl::SyncValue<std::wstring> text_;

	Callback callback_;

public:
	PeerCastManager();
	virtual ~PeerCastManager();

	operator bool() { return peca_; }
	operator std::wstring() { return peca_; }

	bool init(const std::wstring& path);
	void bump();
	void stop();
	void info();

	void setBBSCallback(const boost::function<void()>& callback);
	void setChannelCallback(const boost::function<void(bool)>& callback);

	void getName(std::wstring& name);
	void getContactURL(std::wstring& url);
	void getBitrate(int& bitrate);
	void getInfo(ChannelInfo& info);
	void getInfo(std::wstring& text);

private:
	void bumpThread(const PeerCast& peca);
	void stopThread(const PeerCast& peca);
	void infoThread(const PeerCast& peca);
};

}; // namespace network