#include "stdafx.h"

namespace sl {

void get(const XmlParser& p, std::vector<DecoderInfo>& data)
{
	if (p)
	{
		data.clear();
		p.foreach(L"item", [&](const XmlParser& item){
			DecoderInfo tmp;
			std::wstring id;
			item.get(L"enable", tmp.enable);
			item.get(L"name", tmp.name);
			item.get(L"id", id);

			GUID guid = {};
			CLSIDFromString(id.c_str(), &guid);
			tmp.id = guid;

			data.push_back(tmp);
		});
	}
}

void get(const XmlParser& p, std::vector<bbs::ReplaceElement>& data)
{
	if (p)
	{
		data.clear();
		p.foreach(L"item", [&](const XmlParser& item){
			bbs::ReplaceElement tmp;
			item.get(L"enable", tmp.enable);
			item.get(L"desc", tmp.desc);
			item.get(L"regex", tmp.regex);
			item.get(L"format", tmp.format);
			data.push_back(tmp);
		});
	}
}

void get(const XmlParser& root, DetailConfig& detail)
{
	auto p = root.get(L"detail");

	p = root.get(L"detail.network");
	p.get(L"stop", detail.network.stop);

	p = root.get(L"detail.network.retry");
	p.get(L"timeout", detail.network.retry.timeout);
	p.get(L"count", detail.network.retry.count);
	p.get(L"wait", detail.network.retry.wait);
	p.get(L"stop", detail.network.retry.stop);

	p = root.get(L"detail.network.rebuild");
	p.get(L"enable", detail.network.rebuild.enable);
	p.get(L"count", detail.network.rebuild.count);

	p = root.get(L"detail.network.reader");
	p.get(L"userAgent", detail.network.reader.userAgent);
	p.get(L"timeout", detail.network.reader.timeout);

	p = root.get(L"detail.type.flv");
	p.get(L"loop", detail.type.flv.loop);
	p.get(L"init", detail.type.flv.init);
	p.get(L"offset", detail.type.flv.offset);
	p.get(L"delay", detail.type.flv.delay);
	p.get(L"buffer", detail.type.flv.buffer);
	p.get(L"timestamp", detail.type.flv.timestamp);

	p = root.get(L"detail.decoder");
	p.get(L"dxva2", detail.decoder.dxva2);

	p = root.get(L"detail.video");
	get(p, detail.video);

	p = root.get(L"detail.audio");
	get(p, detail.audio);

	p = root.get(L"detail.browser");
	p.get(L"def", detail.browser.def);
	p.get(L"web", detail.browser.web);
	p.get(L"bbs", detail.browser.bbs);

	p = root.get(L"detail.snapshot");
	p.get(L"folder", detail.snapshot.folder);
	p.get(L"format", detail.snapshot.format);
	p.get(L"jpegQuality", detail.snapshot.jpegQuality);
	p.get(L"display", detail.snapshot.display);
	p.get(L"fit", detail.snapshot.fit);

	p = root.get(L"detail.board");
	p.get(L"contact", detail.board.contact);
	p.get(L"change", detail.board.change);
	p.get(L"userAgent", detail.board.userAgent);
	p.get(L"timeout", detail.board.timeout);
	p.get(L"shitaraba", detail.board.shitaraba);
	p.get(L"reload", detail.board.reload);
	p.get(L"interval", detail.board.interval);
	p.get(L"retry", detail.board.retry);
	p.get(L"thread", detail.board.thread);
	p.get(L"board", detail.board.board);
	p.get(L"move", detail.board.move);
	p.get(L"res", detail.board.res);
	p.get(L"delay", detail.board.delay);
	p.get(L"position", detail.board.position);

	p = root.get(L"detail.board.url");
	get(p, detail.board.url);

	p = root.get(L"detail.board.message");
	get(p, detail.board.message);
	
	p = root.get(L"detail.start");
	p.get(L"recent", detail.start.recent);

	p = root.get(L"detail.end");
	p.get(L"pos", detail.end.pos);
	p.get(L"size", detail.end.size);
	p.get(L"aspect", detail.end.aspect);
	p.get(L"volume", detail.end.volume);
	p.get(L"balance", detail.end.balance);
	p.get(L"mute", detail.end.mute);
}

} // namespace sl
